/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file dac.h
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief header file for DAC utilities. 
*
*******************************************************************************
*
* provide APIs for accessing DAC
******************************************************************************/
#ifndef _DAC_H_
#define _DAC_H_
#ifdef __cplusplus
extern "C" {
#endif
/******************************************************************************
* Includes
******************************************************************************/

/******************************************************************************
* Constants
******************************************************************************/

/* DAC reference select */
enum
{
    DAC_REF_VREFH = 0,
    DAC_REF_VDDA
};

/******************************************************************************
* Macros
******************************************************************************/
#define DACREF_1 VREFH
#define DACREF_2 VDDA
/******************************************************************************
* Types
******************************************************************************/

/*! @brief DAC_CALLBACK function declaration                                  */
typedef void (*DAC_CallbackType)(void);							/*!< ADC call back function */
/*! @} End of DAC_callback                                                    */


/******************************************************************************
* DAC config struct
*
*//*! @addtogroup DAC_configstruct
* @{
*******************************************************************************/
/*!
* @brief DAC module configuration struct.
*
*/

typedef struct 
{

    /* DACx_C0 register setting bit */
    uint8_t bBufferBottomIntEn       :1;     /*!< 1: enable DAC buffer bottom interrupt, 0: disableDAC buffer bottom interrupt */  
    uint8_t bBufferTopIntEn          :1;     /*!< 1: enable DAC buffer top interrupt, 0: disable DAC buffer top interrupt */  
    uint8_t bWatermarkIntEn          :1;     /*!< 1: enable buffer watermark interrupt, 0: disable buffer watermark interrupt */
    uint8_t bLowPowerEn              :1;     /*!< 1: enable low power mode, 0: disable low power mode */
    uint8_t bSoftwareTriggerLaunch   :1;     /*!< write 1 to start DAC when software trigger is enabled */    
    uint8_t bSoftwareTriggerEn       :1;     /*!< 1: DAC software trigger is selected, 0: DAC hadware trigger is selected  */
    uint8_t bDACRefSelect            :1;     /*!< 1: select DACREF_1 as reference, 0: select DACREF_2 as reference  */    
    uint8_t bDACEn                   :1;     /*!< 1: enable DAC, 0:disable DAC */
    /* DACx_C1 register setting bit */
    uint8_t bBufferEn                :1;     /*!< 1: enable DAC buffer, 0:disable DAC buffer */
    uint8_t bBufferModeSelect        :2;     /*!< buffer work mode select */
    uint8_t bWatermarkSelect         :2;     /*!< buffer watermark select */    
    uint8_t bDMAEn                   :1;     /*!< 1: enable DMA transfer, 0:disable DMA transfer */ 
    /* DACx_C2 register setting bit */     
    uint8_t bBufferUpperLimit        :1;     /*!< ?select the upper limit of the DAC buffer */   
    uint8_t bBufferReadPointer       :1;     /*!< ?keep the current value of the buffer read pointer */      
    /* DACx_DATnL register setting bit */       
    uint8_t Data0Low                    ;     /*!< low 8-bit data value */
    uint8_t Data0High                 :4;     /*!< high 4-bit data value */ 
    uint8_t Data1Low                    ;     /*!< low 8-bit data value */
    uint8_t Data1High                 :4;     /*!< high 4-bit data value */ 
    
}DAC_ConfigType, *DAC_ConfigPtr;
/*! @} End of DAC_configstruct                                               */

/******************************************************************************
* Global variables
******************************************************************************/

/*!
 * inline functions
 */
/******************************************************************************
* DAC api list.
*
*//*! @addtogroup DAC_api_list
* @{
*******************************************************************************/
/*****************************************************************************//*!
*
* @brief  enable the DAC module.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_Enable.
*
*****************************************************************************/
__STATIC_INLINE void DAC_Enable(DAC_Type *pDAC)
{
    pDAC->C0 |= DAC_C0_DACEN_MASK ;
}

/*****************************************************************************//*!
*
* @brief  disable the DAC module.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see   DAC_Disable.
*
*****************************************************************************/
__STATIC_INLINE void DAC_Disable(DAC_Type *pDAC)
{
    pDAC->C0 &= ~DAC_C0_DACEN_MASK ;
}

/*****************************************************************************//*!
*
* @brief  enable the DAC buffer bottom interrupt.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_BufferBottomIntEnable.
*
*****************************************************************************/
__STATIC_INLINE void DAC_BufferBottomIntEnable(DAC_Type *pDAC)
{
    pDAC->C0 |= DAC_C0_DACBBIEN_MASK ;
}

/*****************************************************************************//*!
*
* @brief  disable the DAC buffer bottom interrupt.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_BufferBottomIntDisable.
*
*****************************************************************************/
__STATIC_INLINE void DAC_BufferBottomIntDisable(DAC_Type *pDAC)
{
    pDAC->C0 &= ~DAC_C0_DACBBIEN_MASK ;
}

/*****************************************************************************//*!
*
* @brief  enable the DAC buffer top interrupt.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_BufferTopIntEnable.
*
*****************************************************************************/
__STATIC_INLINE void DAC_BufferTopIntEnable(DAC_Type *pDAC)
{
    pDAC->C0 |= DAC_C0_DACBTIEN_MASK ;
}

/*****************************************************************************//*!
*
* @brief  disable the DAC buffer top interrupt.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_BufferTopIntDisable.
*
*****************************************************************************/
__STATIC_INLINE void DAC_BufferTopIntDisable(DAC_Type *pDAC)
{
    pDAC->C0 &= ~DAC_C0_DACBTIEN_MASK ;
}

/*****************************************************************************//*!
*
* @brief  enable the DAC buffer watermark interrupt.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_BufferWatermarkIntEnable.
*
*****************************************************************************/
__STATIC_INLINE void DAC_BufferWatermarkIntEnable(DAC_Type *pDAC)
{
    pDAC->C0 |= DAC_C0_DACBWIEN_MASK ;
}

/*****************************************************************************//*!
*
* @brief  disable the DAC buffer watermark interrupt.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_BufferWatermarkIntDisable.
*
*****************************************************************************/
__STATIC_INLINE void DAC_BufferWatermarkIntDisable(DAC_Type *pDAC)
{
    pDAC->C0 &= ~DAC_C0_DACBWIEN_MASK ;
}

/*****************************************************************************//*!
*
* @brief  get buffer read pointer bottom  flag bit.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_GetBufferBottomFlag.
*
*****************************************************************************/
__STATIC_INLINE uint8_t DAC_GetBufferBottomFlag(DAC_Type *pDAC)
{
    return (pDAC->SR & DAC_SR_DACBFRPBF_MASK);
}

/*****************************************************************************//*!
*
* @brief  clear buffer read pointer bottom flag bit.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_ClrBufferBottomFlag.
*
*****************************************************************************/
__STATIC_INLINE void DAC_ClrBufferBottomFlag(DAC_Type *pDAC)
{
    pDAC->SR &= ~DAC_SR_DACBFRPBF_MASK;
}

/*****************************************************************************//*!
*
* @brief  get buffer read pointer top flag bit.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_GetBufferTopFlag.
*
*****************************************************************************/
__STATIC_INLINE uint8_t DAC_GetBufferTopFlag(DAC_Type *pDAC)
{
    return (pDAC->SR & DAC_SR_DACBFRPTF_MASK);
}

/*****************************************************************************//*!
*
* @brief  clear buffer read pointer top flag bit.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_ClrBufferTopFlag.
*
*****************************************************************************/
__STATIC_INLINE void DAC_ClrBufferTopFlag(DAC_Type *pDAC)
{
    pDAC->SR &= ~DAC_SR_DACBFRPTF_MASK;
}

/*****************************************************************************//*!
*
* @brief  get buffer read pointer watermark flag bit.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_GetBufferWatermarkFlag.
*
*****************************************************************************/
__STATIC_INLINE uint8_t DAC_GetBufferWatermarkFlag(DAC_Type *pDAC)
{
    return (pDAC->SR & DAC_SR_DACBFWMF_MASK);
}

/*****************************************************************************//*!
*
* @brief  clear buffer read pointer watermark flag bit.
*        
* @param[in]  pDAC      pointer to an DAC module.
*              
* @return none.
*
* @ Pass/ Fail criteria: none.
*
* @see    DAC_ClrBufferWatermarkFlag.
*
*****************************************************************************/
__STATIC_INLINE void DAC_ClrBufferWatermarkFlag(DAC_Type *pDAC)
{
    pDAC->SR &= ~DAC_SR_DACBFWMF_MASK;
}

/******************************************************************************
* Global functions
******************************************************************************/
void DAC_Init(DAC_Type *pDAC, DAC_ConfigType *pConfig);
void DAC_DeInit(DAC_Type *pDAC); 
void DAC_SetCallback(DAC_Type *pDAC, DAC_CallbackType pfnCallback);
/*! @} End of DAC_api_list                                                  */
#ifdef __cplusplus
}
#endif
#endif /* _DAC_H_ */












































































































































