/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file dma.c
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief providing APIs for configuring eDMA module (eDMA). 
*
*******************************************************************************
*
* provide APIs for configuring eDMA module (eDMA).
******************************************************************************/
#include "common.h"
#include "dma.h"

/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Constants and macros
******************************************************************************/

/******************************************************************************
* Local types
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* Local variables
******************************************************************************/
static DMA_CallbackType DMA_Callback[4] = {(DMA_CallbackType)NULL};
/******************************************************************************
* Local functions
******************************************************************************/
void DMA0_Isr( void );
void DMA1_Isr( void );
void DMA2_Isr( void );
void DMA3_Isr( void );

/******************************************************************************
* Global functions
******************************************************************************/


/******************************************************************************
* define DMA APIs
*
*//*! @addtogroup dma_api_list
* @{
*******************************************************************************/
/*****************************************************************************//*!
   *
   * @brief Initialize DMA module.
   *        
   * @param[in] pDMAx      point to DMA module type.
 	 * @param[in] pDMAConfig point to DMA configure structure.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/

void DMA_Init(DMA_Type *pDMAx,DMA_ConfigType *pDMAConfig)
{
   DMA_ChannelEn(pDMAx,pDMAConfig->Channel,pDMAConfig->Slot);

    /* Clear error status bit */
   DMA_ClearEEIReg(pDMAx,pDMAConfig->Channel);
   /* Clear Done flag */
   DMA_ClearDONEbit(pDMAx,pDMAConfig->Channel);
   
   if(pDMAConfig->sSetting.bAsysReqEn)
   {
		DMA_AsynReqEn(pDMAx,0x01<<pDMAConfig->Channel);
   }
   if(pDMAConfig->sSetting.bDebugEn)
   {
		DMA_DebugEn(pDMAx);
   }
   if(pDMAConfig->sSetting.bErrorIntEn)
   {
		DMA_SetEEIReg(pDMAx,pDMAConfig->Channel);
   }
   if(pDMAConfig->sSetting.bHOE)
   {
		DMA_HaltDMAOnError(pDMAx);
   }
   if(pDMAConfig->sSetting.bIntEn)
   {
		DMA_WriteINTReg(pDMAx,0x01<<pDMAConfig->Channel);
   }
   if(pDMAConfig->sSetting.bRoundRobinEn)
   {
		DMA_ChannelArbSelection(pDMAx,Round_Robin_Arbitration);
   }

   DMA_UpdatedTCDInfo(pDMAx,pDMAConfig->Channel,&pDMAConfig->sTCDInfo);

   /* enable request */
   if(pDMAConfig->sSetting.bReqEn)
   {
        DMA_SetERQReg(pDMAx,pDMAConfig->Channel);
   }
}

/*****************************************************************************//*!
   *
   * @brief updated TCD information.
   *        
   * @param[in] pDMAx      	point to DMA module type.
   * @param[in] pTCD   		point to TCD configure structure.
   * @param[in] Channel   	point to DMA channel number.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
void DMA_UpdatedTCDInfo(DMA_Type *pDMAx,uint8_t Channel,DMA_TCDType *pTCD)
{
	memcpy((void *)&pDMAx->TCD[Channel],(void *)pTCD,sizeof(DMA_TCDType));
}
/*****************************************************************************//*!
   *
   * @brief set DMA channel priority.
   *        
   * @param[in] pDMAx      point to DMA module type.
   * @param[in] pTCD   point to TCD configure structure.
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
*****************************************************************************/
void DMA_SetChannelPriority(DMA_Type *pDMAx,uint8_t Channel,uint8_t Priority)
{
	switch(Channel)
	{
	case 0:
		pDMAx->DCHPRI0 = Priority;
		break;
	case 1:
		pDMAx->DCHPRI1 = Priority;
		break;
	case 2:
		pDMAx->DCHPRI2 = Priority;
		break;
	case 3:
		pDMAx->DCHPRI3 = Priority;
		break;
	default:
		break;
	}
}

/*****************************************************************************//*!
   *
   * @brief set call back function for DMA module.
   *        
   * @param[in] pCallBack 	point to address of DMA call back function.
   * @param[in] Channel 	DMA channel number.
   *
   * @return none.
   *
   * @ Pass/ Fail criteria:  none.
*****************************************************************************/

void DMA_SetCallBack( DMA_CallbackType pCallBack,uint8_t Channel )
{
    DMA_Callback[Channel] = pCallBack;
}
/*! @} End of dma_api_list                                               						*/


/*****************************************************************************//*!
   *
   * @brief DMA interrupt service routine.
   *        
   * @param  
   *
   * @return none
   *
   * @ Pass/ Fail criteria:  none
*****************************************************************************/
void DMA0_Isr( void )
{
    if( DMA_Callback[0] )
    {
        DMA_Callback[0]();
    }
}
/*****************************************************************************//*!
   *
   * @brief DMA interrupt service routine.
   *        
   * @param  
   *
   * @return none
   *
   * @ Pass/ Fail criteria:  none
*****************************************************************************/
void DMA1_Isr( void )
{
    if( DMA_Callback[1] )
    {
        DMA_Callback[1]();
    }
}
/*****************************************************************************//*!
   *
   * @brief DMA interrupt service routine.
   *        
   * @param  
   *
   * @return none
   *
   * @ Pass/ Fail criteria:  none
*****************************************************************************/
void DMA2_Isr( void )
{
    if( DMA_Callback[2] )
    {
        DMA_Callback[2]();
    }
}

/*****************************************************************************//*!
   *
   * @brief DMA interrupt service routine.
   *        
   * @param  
   *
   * @return none
   *
   * @ Pass/ Fail criteria:  none
*****************************************************************************/
void DMA3_Isr( void )
{
    if( DMA_Callback[3] )
    {
        DMA_Callback[3]();
    }
}

