
/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file mmdvsq.h
*
* @author Freescale
*
* @version 0.0.1
*
* @date Aug.26, 2013
*
* @brief header file for MMDVSQ utilities. 
*
*******************************************************************************
*
* provide h/w divide and Sqrt APIs with MMDVSQ
******************************************************************************/

#ifndef MMDVSQ_H_
#define MMDVSQ_H_

#include "common.h"

#ifdef __cplusplus
extern "C" {
#endif
/******************************************************************************
* Includes
******************************************************************************/

/******************************************************************************
* Constants
******************************************************************************/

/******************************************************************************
* Macros
******************************************************************************/


/*!****************************************************************************
* Types
******************************************************************************/
    
/* MMDVSQ divide configuration
 */  
/******************************************************************************
* define divide configuration structure
*
*//*! @addtogroup mmdvsq_divide_config_type
* @{
*******************************************************************************/
 
/*!
 * @brief MMDVSQ divide config type.
 *
 */
typedef struct
{
	uint8_t bUsignedDivide          : 1;        /*!< 1: perform unsigned divide, 0: perform signed divide */
	uint8_t bCalculateRemainder     : 1;        /*!< 1: calculate remainder, 0: calculate quotient */
	uint8_t bDisableFastStart       : 1;        /*!< 1: disable fast start, 0: enable fast start */
	uint8_t bEnableDivideBy0        : 1;        /*!< 1: enable divide by 0, 0: disable divide by 0 */
	uint8_t bRsvd                   : 4;        /*!< reserved bits */
} MMDVSQ_DivideConfigType, *MMDVSQ_DivideConfigPtr;
/*! @} End of mmdvsq_divide_config_type                        					*/



/*****************************************************************************//*!
*
* @brief perform h/w sqrt for unsigned data.
*        
* @param[in]  u32Data  unsigned data 
*
* @return unsigned 16-bit data
*
* @ Pass/ Fail criteria: none
* @see  MMDVSQ_DivSQ
*****************************************************************************/

__STATIC_INLINE uint16_t    MMDVSQ_Sqrt(uint32_t    u32Data)
{
    uint16_t    u16Result;    
    MMDVSQ->RCND = u32Data;
    u16Result  = MMDVSQ->RES;
    return (u16Result);
}

/*****************************************************************************//*!
*
* @brief perform h/w signed divide and return quotient.
*           
* @param[in]  i32Numerator  signed numerator 
* @param[in]  i32Denominator  signed denominator 
*
* @return signed 32-bit quotient
*
* @ Pass/ Fail criteria: none
* @warning This routine does not check whether denominator is 0 or not, users need
*       check this status by calling MMDVSQ_IsDivideBy0().
* @see  MMDVSQ_DivSR
*****************************************************************************/

__STATIC_INLINE int32_t    MMDVSQ_DivSQ(int32_t    i32Numerator, int32_t i32Denominator)
{
    uint32_t    u32Result;
    /*
     * If CSR[REM, USGN] = 0b00, perform a signed divide, returning the quotient
     * If CSR[REM, USGN] = 0b01, perform an unsigned divide, returning the quotient
     * If CSR[REM, USGN] = 0b10, perform a signed divide, returning the remainder
     * If CSR[REM, USGN] = 0b11, perform an unsigned divide, returning the remainder
     */ 
//    uint32_t    csr = MMDVSQ->CSR;  /* save control bits */
    MMDVSQ->CSR &= ~(MMDVSQ_CSR_REM_MASK | MMDVSQ_CSR_USGN_MASK);
    MMDVSQ->DEND = i32Numerator;
    MMDVSQ->DSOR = i32Denominator;
    u32Result  = MMDVSQ->RES;
    
    /* restore the control bits */
//    MMDVSQ->CSR = csr;
    
    return (u32Result);
}


/*****************************************************************************//*!
*
* @brief perform h/w unsigned divide and return quotient.
*           
* @param[in]  u32Numerator  unsigned numerator 
* @param[in]  u32Denominator  unsigned denominator 
*
* @return usigned 32-bit quotient
*
* @ Pass/ Fail criteria: none
* @warning This routine does not check whether denominator is 0 or not, users need
*       check this status by calling MMDVSQ_IsDivideBy0().
* @see  MMDVSQ_DivUR
*****************************************************************************/

__STATIC_INLINE uint32_t    MMDVSQ_DivUQ(uint32_t    u32Numerator, uint32_t u32Denominator)
{
    uint32_t    u32Result;
    /*
     * If CSR[REM, USGN] = 0b00, perform a signed divide, returning the quotient
     * If CSR[REM, USGN] = 0b01, perform an unsigned divide, returning the quotient
     * If CSR[REM, USGN] = 0b10, perform a signed divide, returning the remainder
     * If CSR[REM, USGN] = 0b11, perform an unsigned divide, returning the remainder
     */ 
//    uint32_t    csr = MMDVSQ->CSR;  /* save control bits */
    MMDVSQ->CSR = (MMDVSQ->CSR & ~(MMDVSQ_CSR_REM_MASK)) | MMDVSQ_CSR_USGN_MASK;
    MMDVSQ->DEND = u32Numerator;
    MMDVSQ->DSOR = u32Denominator;
    u32Result  = MMDVSQ->RES;
    
    /* restore the control bits */
//    MMDVSQ->CSR = csr;
    
    return (u32Result);    
}

/*****************************************************************************//*!
*
* @brief perform h/w signed divide and return remainder.
*           
* @param[in]  i32Numerator  signed numerator 
* @param[in]  i32Denominator  signed denominator 
*
* @return signed 32-bit remainder
*
* @ Pass/ Fail criteria: none
* @warning This routine does not check whether denominator is 0 or not, users need
*       check this status by calling MMDVSQ_IsDivideBy0().
* @see  MMDVSQ_DivUR
*****************************************************************************/

__STATIC_INLINE int32_t    MMDVSQ_DivSR(int32_t    i32Numerator, int32_t i32Denominator)
{
    int32_t    i32Result;
    /*
     * If CSR[REM, USGN] = 0b00, perform a signed divide, returning the quotient
     * If CSR[REM, USGN] = 0b01, perform an unsigned divide, returning the quotient
     * If CSR[REM, USGN] = 0b10, perform a signed divide, returning the remainder
     * If CSR[REM, USGN] = 0b11, perform an unsigned divide, returning the remainder
     */ 
//    uint32_t    csr = MMDVSQ->CSR;  /* save control bits */
    MMDVSQ->CSR = (MMDVSQ->CSR & ~(MMDVSQ_CSR_USGN_MASK)) | MMDVSQ_CSR_REM_MASK;
    MMDVSQ->DEND = i32Numerator;
    MMDVSQ->DSOR = i32Denominator;
    i32Result  = MMDVSQ->RES;
    
    /* restore the control bits */
//    MMDVSQ->CSR = csr;
    
    return (i32Result);          
}


/*****************************************************************************//*!
*
* @brief perform h/w unsigned divide and return remainder.
*           
* @param[in]  u32Numerator  unsigned numerator 
* @param[in]  u32Denominator  unsigned denominator 
*
* @return unsigned 32-bit remainder
*
* @ Pass/ Fail criteria: none
* @warning This routine does not check whether denominator is 0 or not, users need
*       check this status by calling MMDVSQ_IsDivideBy0().
* @see  MMDVSQ_DivUQ
*****************************************************************************/

__STATIC_INLINE uint32_t    MMDVSQ_DivUR(uint32_t    u32Numerator, uint32_t u32Denominator)
{
    uint32_t    u32Result;
    /*
     * If CSR[REM, USGN] = 0b00, perform a signed divide, returning the quotient
     * If CSR[REM, USGN] = 0b01, perform an unsigned divide, returning the quotient
     * If CSR[REM, USGN] = 0b10, perform a signed divide, returning the remainder
     * If CSR[REM, USGN] = 0b11, perform an unsigned divide, returning the remainder
     */ 
 //   uint32_t    csr = MMDVSQ->CSR;  /* save control bits */
    MMDVSQ->CSR |= (MMDVSQ_CSR_REM_MASK | MMDVSQ_CSR_USGN_MASK);
    MMDVSQ->DEND = u32Numerator;
    MMDVSQ->DSOR = u32Denominator;
    u32Result  = MMDVSQ->RES;
    
    /* restore the control bits */
  //  MMDVSQ->CSR = csr;
    
    return (u32Result);      
}


/*****************************************************************************//*!
*
* @brief check h/w divide to see if the denominator is 0.
*           
* @param none 
*
* @return unsigned 8-bit boolean value
*
* @ Pass/ Fail criteria: none
* @see  MMDVSQ_DivUQ
*****************************************************************************/

__STATIC_INLINE uint8_t    MMDVSQ_IsDivideBy0(void)
{
    uint8_t    u8IsDivideBy0;
    
    u8IsDivideBy0 = (MMDVSQ->CSR & MMDVSQ_CSR_DZ_MASK)? TRUE: FALSE;

    return (u8IsDivideBy0);      
}

/*****************************************************************************//*!
*
* @brief start divide manually.
*           
* @param none 
*
* @return none
*
* @ Pass/ Fail criteria: none
* @see  MMDVSQ_DivUQ
*****************************************************************************/

__STATIC_INLINE void    MMDVSQ_StartDivide(void)
{
    
    MMDVSQ->CSR |= MMDVSQ_CSR_SRT_MASK;

}

/******************************************************************************
* Global functions
******************************************************************************/

void    MMDVSQ_ConfigDivide(MMDVSQ_DivideConfigPtr pDivideConfig);

#ifdef __cplusplus
}
#endif

#endif  /* MMDVSQ_H_ */



