/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2004-2011 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
******************************************************************************
*
* @file      main.c   
*
* @author    Petr Staszko
* 
* @version   1.0.0.0
*
* @date      15.8-2013
* 
* @brief     Sensorless BLDC motor control application
*
* @remark    
*
*******************************************************************************
*
* The main file of Sensorless BLDC motor control application for Kinetis KV10
*
******************************************************************************/

#include "main.h"
#include "BLDC_appconfig.h"
#include "peripherals_init.h"
#include "freemaster.h"
#include "M1_StateMachine.h" /* Motor 1's state machine */
#include "state_machine.h"
#include "motor_structure.h"

/******************************************************************************
* Global variables
******************************************************************************/
extern MCSTRUC_BLDC_SNLS_INT_T gsM1_Drive;
volatile UInt16 uw16TriggerErrorCnt;

/******************************************************************************
* Global functions
******************************************************************************/
void UpdateBldcHwModules(MCSTRUC_BLDC_SNLS_INT_T *);
extern void ReadSwFaults(MCSTRUC_BLDC_SNLS_INT_T *);
extern void TimeEvent(MCSTRUC_BLDC_SNLS_INT_T *);
extern void FastControlLoop(MCSTRUC_BLDC_SNLS_INT_T *);
extern void SlowControlLoop(MCSTRUC_BLDC_SNLS_INT_T *);
extern void Demonstration(MCSTRUC_BLDC_SNLS_INT_T *);

/******************************************************************************
* Local functions
******************************************************************************/
int main(void);

int main(void)
{
  DisableInterrupts;
  MCU_init();

  FMSTR_Init();
  SM_StateMachine(&gsM1_Ctrl);          // Application variables init
  
  /* Enable Interrupts globally */
  EnableInterrupts;
  
  M1_SetAppSwitch(1);
    
    /* infinite loop */
    while(1)
    {
      FMSTR_Poll();
    }
}

/***************************************************************************//*!
 * @brief   ADC Callback function
 *
 * Description:
 * Three analogue signals are measured by the ADC module
 *
 *   1) DC Bus current  - triggered by H/W
 *   2) Phase voltage   - triggered by H/W (Back to back PDB mode)
 *   3) DC Bus voltage  - triggered by SW
 * 
 * After second ADC sample is converted (2), ADC interrupt service routine
 * is executed.
 * PDB module is disabled (to avoid error due to Back-to-back mode of PDB)
 * ADC is configured to S/W triggered mode
 * Third ADC sample is converted (3) (ADC started by S/W trigger, pool mode)
 * PDB module is enabled
 * ADC is configured to H/W triggered mode
 * ADC channels are set back for next measurements
 * 
 ******************************************************************************/
void ADC0_Isr(void)
{
  // Read timer counter and value registers
  gsM1_Drive.uw16TimeCurrent      = FTM1->CNT;
  gsM1_Drive.uw16TimeCurrentEvent = FTM1->CONTROLS[0].CnV;
  
  // Read ADC results
  // (ADC triggered by H/W trigger from PDB)
  gsM1_Drive.f16DcBusCurrentRaw = ADC0->R[0] << 3;            // 15-bit positive number
  gsM1_Drive.f16PhaseVoltage = ADC0->R[1] << 3;            // left justified 12-bit result

  // - disable PDB module to avoid error due to
  //   Back-to-back mode of PDB
  // - configure ADC to S/W trigger mode
  PDB0->CH[0].C1 = 0;                                   // disable PDB channel 0
  ADC0->SC2 &= ~ADC_SC2_ADTRG_MASK;                      // set ADC to S/W trigger

  // - start of conversion by S/W, wait for end
  // - read ADC result
  ADC0->SC1[0] = ADC_CHANNEL_DCBV;
  while (!(ADC0->SC1[0] & ADC_SC1_COCO_MASK) ) {}
  gsM1_Drive.f16DcbVoltage = ADC0->R[0] << 3;              // left justified 12-bit result
  
  // - enable PDB module
  // - configure ADC to H/W trigger mode
  PDB0->CH[0].C1 = PDB_C1_TOS(3) | PDB_C1_EN(3) | PDB_C1_BB(2); // enable PDB channel 0
  ADC0->SC2 |= ADC_SC2_ADTRG_MASK;                       // set to H/W trigger
  
  // Configure ADC channel A for next measurement
  ADC0->SC1[0] = ADC_CHANNEL_DCBI;

  gsM1_Drive.f16DcBusCurrentNoFiltered = MLIB_ShLSat_F16(gsM1_Drive.f16DcBusCurrentRaw - gsM1_Drive.f16DcBusCurrentOffset, 1);
  // Filtered DC Bus current value
  gsM1_Drive.f16DcBusCurrent = GDFLIB_FilterMA_F16(gsM1_Drive.f16DcBusCurrentNoFiltered, &gsM1_Drive.trCurrentMaFilter);
  gsM1_Drive.f16PhaseBemf = gsM1_Drive.f16PhaseVoltage - (gsM1_Drive.f16DcbVoltage >> 1);

  FMSTR_Recorder();     // Back-EMF voltage (must be located before FastControlLoop, because it is modified in FastControlLoop)
  
  FastControlLoop(&gsM1_Drive);
  
  // Configure ADC channel B for next measurement
  ADC0->SC1[1] = bldcAdcSectorCfg[gsM1_Drive.w16CmtSector] | ADC_SC1_AIEN_MASK;  // w16CmtSector may be modified in FastControlLoop(...)

  ReadSwFaults(&gsM1_Drive);

  SM_StateMachine(&gsM1_Ctrl);

  UpdateBldcHwModules(&gsM1_Drive);
}

/***************************************************************************//*!
 * @brief   FTM1 interrupt service routine
 * - Forced commutation control
 ******************************************************************************/
void FTM1_Isr(void)
{
  // Read timer counter and value registers
  gsM1_Drive.uw16TimeCurrent      = FTM1->CNT;
  gsM1_Drive.uw16TimeCurrentEvent = FTM1->CONTROLS[0].CnV;
  
  TimeEvent(&gsM1_Drive);
  
  SM_StateMachine(&gsM1_Ctrl);
  
  UpdateBldcHwModules(&gsM1_Drive);
  
  FTM1->CONTROLS[0].CnSC &= ~FTM_CnSC_CHF_MASK;
}

/***************************************************************************//*!
 * @brief   FTM2 interrupt service routine
 * - slow control loop
 * - Low priority ISR
 ******************************************************************************/
void FTM2_Isr(void)
{
  SlowControlLoop(&gsM1_Drive);

  MC33927_readStatus();
  
  Demonstration(&gsM1_Drive);                          //only for demo demonstration

  FTM2->CONTROLS[0].CnV += SLOW_TIMER_PERIOD;
  FTM2->CONTROLS[0].CnSC &= ~FTM_CnSC_CHF_MASK;
}

/***************************************************************************//*!
 * @brief   PDB0 interrupt service routine
 * Trigger error clear routine
 ******************************************************************************/
void PDB0_Isr(void)
{
  if (PDB0->CH[0].S & PDB_S_ERR_MASK)
  {
    if (uw16TriggerErrorCnt < 0xffff) uw16TriggerErrorCnt++;
    PDB0->SC &= ~PDB_SC_PDBEN_MASK;      // disable PDB
    PDB0->CH[0].S &= ~PDB_S_ERR_MASK;
    PDB0->SC |= PDB_SC_PDBEN_MASK;       // Enable PDB
  }
}

/***************************************************************************//*!
 * @brief   Update of W/H modules used by BLDC algorithm based on request flags
 ******************************************************************************/
void UpdateBldcHwModules(MCSTRUC_BLDC_SNLS_INT_T * sM_Drive)
{
  // Disable PWM outputs
  // If uw16TriggerErrorCnt > 1 then there is probably issue with ADC ISR (it is probably not executed), therefore PWM outputs are disabled to protect output MOSFETs; 1 error is allowed.
  // "uw16TriggerErrorCnt > 1" condition should be deleted in the final application.
  if ((sM_Drive->uw16HwUpdateRequests & UPDATE_PWM_DISABLE_REQ) | (uw16TriggerErrorCnt > 1))
  {
    sM_Drive->uw16HwUpdateRequests &= ~(UPDATE_PWM_DISABLE_REQ | UPDATE_BRAKE_RESISTOR_ON_REQ | UPDATE_DUTYCYCLE_REQ | UPDATE_PWM_CONFIG_REQ | UPDATE_PWM_ALIGNMENT_REQ);
    FTM0_SetPwmOutput(7);                     // Disable PWM outputs
    BRAKE_RESISTOR_OFF;
  }

  // Request to set new time event (update timer value register)
  if (sM_Drive->uw16HwUpdateRequests & UPDATE_TIME_EVENT_REQ)
  {
    sM_Drive->uw16HwUpdateRequests &= ~UPDATE_TIME_EVENT_REQ;
    FTM1->CONTROLS[0].CnV = sM_Drive->uw16TimeNextEvent;
  }
  
  // Update PWM duty cycle if requested
  if (sM_Drive->uw16HwUpdateRequests & UPDATE_DUTYCYCLE_REQ)
  {
    sM_Drive->uw16HwUpdateRequests &= ~UPDATE_DUTYCYCLE_REQ;
    FTM0_SetDutyCycle(sM_Drive->f16DutyCycle);
  }

  // Apply commutation vector (alternatively enable PWM outputs)
  if (sM_Drive->uw16HwUpdateRequests & UPDATE_PWM_CONFIG_REQ)
  {
    sM_Drive->uw16HwUpdateRequests &= ~UPDATE_PWM_CONFIG_REQ;
    FTM0_SetPwmOutput(sM_Drive->w16CmtSector);             // apply commutation vector    
  }
   
  // Apply alignment vector
  if (sM_Drive->uw16HwUpdateRequests & UPDATE_PWM_ALIGNMENT_REQ)
  {
    sM_Drive->uw16HwUpdateRequests &= ~UPDATE_PWM_ALIGNMENT_REQ;
    FTM0_SetPwmOutput(6);                    // Apply alignment vector
  }
  
  // Brake resistor
  if (sM_Drive->uw16HwUpdateRequests & UPDATE_BRAKE_RESISTOR_ON_REQ) {
    BRAKE_RESISTOR_ON;
  }
  else {
    BRAKE_RESISTOR_OFF;
  }
}