
/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file Thermistor_lab.c
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun. 25, 2013
*
* @brief providing thermistor lab. 
*
*******************************************************************************/

#include "common.h"
#include "mcg.h"
#include "uart.h"
#include "kv10_config.h"
#include "ftm.h"
#include "adc.h"
#include "pdb.h"
#include "arm_math.h" 
#include "math_helper.h" 
/* include FreeMASTER Serial Driver functions */
#include "freemaster.h"

/******************************************************************************
* Constants and macros
******************************************************************************/

/* PWM frequency */
#define PWM_FREQ_HZ          16000

/* use const to output FIR filter coefficients
 to the data section to be placed in data memory.*/
#include "FirCoefs.h" 

/* low pass filter */
#define DIFF_TS     400
#define BUFF_SIZE   20                              /* filter buf size */

/* S/W timer */
#define SWTMR_TOUT  ((400*PWM_FREQ_HZ)/1000)        /* 400 ms period */
#define _100MS      ((100*PWM_FREQ_HZ)/1000)        /* 100 ms period */

/* RT thresholds */
#define RT1ONTH     -8  
#define RT1OFFTH    8  
#define RT2ONTH     -25 
#define RT2OFFTH    20 
#define RT3ONTH     -10  
#define RT3OFFTH    8 
#define RT4ONTH     -25 
#define RT4OFFTH    10 

/* FIR algorithm related */
#define Q15                     32768
#define FRAC16(a)               (a*Q15)

#define TEST_LENGTH_SAMPLES     1 
#define BLOCK_SIZE              1 
#define NUM_TAPS                31 

/* ADC channels definition for thermistors using differential input*/
#define ADC0_CHANNEL0         1          /*!< RT1, diff -> ADC0_SE1/PTE16/ADC1_SE0/ADC0_DP1 */ 
#define ADC0_CHANNEL1         0          /*!< RT3, diff -> ADC0_SE0/PTE20/ADC0_DP0 */
#define ADC1_CHANNEL0         1          /*!< RT2, diff -> ADC0_SE6/PTE18/ADC1_DP1 */ 
#define ADC1_CHANNEL1         2          /*!< RT4, diff -> PTB3/ADC1_SE2_OTHER/ADC1_DP2 */


/* MAX no of ADC samples */
#define MAX_NO_SAMPLES      16  //256

/******************************************************************************
* Global variables
******************************************************************************/
typedef union
{
    struct
    {
        uint16_t rt1on          : 1; /*!< RT1ON */
        uint16_t rt2on          : 1; /*!< RT2ON */
        uint16_t rt3on          : 1; /*!< RT3ON */
        uint16_t rt4on          : 1; /*!< RT4ON */
        uint16_t idleloop       : 1; /*!< idle loop flag */
        uint16_t Rsvd           : 11; /*!< RESERVED */
    } Bits;
    uint16_t W16;
} RT_ControlType;                  /* RT Control bits */


/* Global variables */
volatile uint16_t        swDiffCNTR, swTimerCNTR;
volatile uint16_t        timeout;

volatile RT_ControlType rton;
volatile uint16_t        rt_filter_on;
volatile uint16_t        rt_filter_off;

volatile int16_t         rt1_filt_buff[BUFF_SIZE], rt2_filt_buff[BUFF_SIZE];
volatile int16_t         rt3_filt_buff[BUFF_SIZE], rt4_filt_buff[BUFF_SIZE];

volatile uint16_t        i_delay = 1, i_sample = 0;

int16_t rt1, rt2, rt3, rt4;
int16_t rt1_filt, rt2_filt, rt3_filt, rt4_filt;
int16_t delta_rt1, delta_rt2;
int16_t delta_rt3, delta_rt4;

/* 
 * Declare FIR State buffer of size (numTaps + u32BlockSize - 1) 
 */ 

static int16_t i16FirStateQ15_1[BLOCK_SIZE + NUM_TAPS - 1];
static int16_t i16FirStateQ15_2[BLOCK_SIZE + NUM_TAPS - 1];
static int16_t i16FirStateQ15_3[BLOCK_SIZE + NUM_TAPS - 1];
static int16_t i16FirStateQ15_4[BLOCK_SIZE + NUM_TAPS - 1];

/*
 * instances of FIR internal state
 */
static arm_fir_instance_q15 S1,S2,S3,S4; 

uint32_t u32BlockSize = BLOCK_SIZE; 
uint32_t u32NumBlocks = TEST_LENGTH_SAMPLES/BLOCK_SIZE; 

/* for ADC samples */
uint16_t u16Result0A[MAX_NO_SAMPLES]={0},u16Result0B[MAX_NO_SAMPLES]={0},u16Result1A[MAX_NO_SAMPLES]={0},u16Result1B[MAX_NO_SAMPLES]={0};
volatile uint16_t u8CycleTimes=0,gu8RdCounter =0;

extern int core_clk_khz;      /* core clock frequency in KHz */

/******************************************************************************
* Local types
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* Local variables
******************************************************************************/

/******************************************************************************
* Local functions
******************************************************************************/
int main (void);
void ADC0_Task(void);
void ADC1_Task(void);
void FTM_PinConfig(void);
void RT_LEDs_On(void);
void RT_LEDs_OffAll(void);
void RT1_3_LED_On();
void RT1_3_LED_Off();
void RT2_4_LED_On();
void RT2_4_LED_Off();

void RT_Check(int16_t i16Rt1, int16_t i16Rt2, int16_t i16Rt3, int16_t i16Rt4);


/******************************************************************************
* Global functions
******************************************************************************/

/********************************************************************/
int main (void)
{
    ADC_ConfigType  sADC_Config  = {0}; 
    ADC_ConfigType  *pADC_Config = &sADC_Config;
    PDB_ConfigType  sPDB_Config  = {0};
    PDB_ConfigType  *pPDB_Config = &sPDB_Config;
    arm_status err;
    uint16_t timeout100MS;

#if !defined(USE_TERMINAL)    
    FMSTR_Init();       /*! initialize freemaster */
    /* enable UART0 interrupt */
    NVIC_EnableIRQ(UART0_IRQn);
#else
 	printf("\nRunning the PWM_PDB_ADC_Lab project.\n");
#endif
   
    /* initiliaze LEDs to OFF */
    LED0_Init();
    LED1_Init();
    LED2_Init();
    LED3_Init();
    LED4_Init();
    LED5_Init();
    LED6_Init();
    LED7_Init();
    RT_LEDs_OffAll();
    
    /* Call FIR init function to initialize the instance structure. */
    err = arm_fir_init_q15(&S1, NUM_TAPS, (q15_t*)&i16FirCoefs[0], &i16FirStateQ15_1[0], u32BlockSize); 
    ASSERT(err == ARM_MATH_SUCCESS);
    err = arm_fir_init_q15(&S2, NUM_TAPS, (q15_t*)&i16FirCoefs[0], &i16FirStateQ15_2[0], u32BlockSize); 
    ASSERT(err == ARM_MATH_SUCCESS);
    err = arm_fir_init_q15(&S3, NUM_TAPS, (q15_t*)&i16FirCoefs[0], &i16FirStateQ15_3[0], u32BlockSize); 
    ASSERT(err == ARM_MATH_SUCCESS);
    err = arm_fir_init_q15(&S4, NUM_TAPS, (q15_t*)&i16FirCoefs[0], &i16FirStateQ15_4[0], u32BlockSize); 
    ASSERT(err == ARM_MATH_SUCCESS);
    

    /* ADC module configuration */
    /* system clock is75MHz, bus clock is 25MHz */    
    /* enable clock 5 divider and set divider=4, the clock frequency is system clock/4=18.75MHz */
    SIM->CLKDIV1 |= SIM_CLKDIV1_OUTDIV5EN_MASK | SIM_CLKDIV1_OUTDIV5(0x03);/*!<out5 divider is 4, ADC clock source is 75/4MHz*/ 
    pADC_Config->bClocklSelect      = 0x02; /*!<clock source is alternative clock*/
    pADC_Config->bConversionMode    = 0x03; /*!<16-bit single-end mode*/ 
    pADC_Config->bHardwareTriggerEn = TRUE; /*!<enble ADC hardware trigger*/
    pADC_Config->bIntEn             = TRUE; /*!<ADC0 and ADC1 interrupt enable*/ 
    pADC_Config->bDiffEn            = TRUE; /*!< diff input mode */
    ADC_Cal(ADC0);                          /*!<ADC0 calibration*/
    ADC_Cal(ADC1);                          /*!<ADC0 calibration*/
    ADC_Init(ADC0, pADC_Config);            /*!<inital ADC0 */
    ADC_Init(ADC1, pADC_Config);            /*!<inital ADC1*/
    ADC_SetCallBack(0, ADC0_Task);
    ADC_SetCallBack(1, ADC1_Task);    
    
    /* write ADC0 channel and ADC1 channel in differential mode */
    ADC_SetChannel(ADC0,0,ADC0_CHANNEL0, TRUE); 
    ADC_SetChannel(ADC0,1,ADC0_CHANNEL1, TRUE);
    ADC_SetChannel(ADC1,0,ADC1_CHANNEL0, TRUE);
    ADC_SetChannel(ADC1,1,ADC1_CHANNEL1, TRUE);

    /* PDB module configuration */
    /* PDB clock is bus clock, ~25MHz */
    /* trigger delay for ping-pong buffers in both ADCs are (1000-200)/25=32uS */
    pPDB_Config->bPDBEn             =    TRUE; /*!<enable PDB*/  
    pPDB_Config->u16ModulusValue    =    1500; /*!<set modulo value*/ 
    pPDB_Config->u16InterruptDelay  =    1500; /*!<set interrupt delay value, it is not necessary if disable interrupt*/
    pPDB_Config->u16Ch0Delay0       =    200;  /*!<set channel 0 pretrigger 0 delay time*/
    pPDB_Config->u16Ch0Delay1       =    1000; /*!<set channel 0 pretrigger 1 delay time*/
    pPDB_Config->u16Ch1Delay0       =    200;  /*!<set channel 1 pretrigger 0 delay time*/
    pPDB_Config->u16Ch1Delay1       =    1000; /*!<set channel 1 pretrigger 1 delay time*/    
    pPDB_Config->u8Ch0PretriggerEn  =    0x03; /*!<enable channel 0 pretrigger */
    pPDB_Config->u8Ch1PretriggerEn  =    0x03; /*!<enable channel 1 pretrigger */    
    pPDB_Config->u8Ch0PretriggerOut =    0x03; /*!<enable channel 0 pretrigger output */
    pPDB_Config->u8Ch1PretriggerOut =    0x03; /*!<enable channel 1 pretrigger output */
    /*channel delay is not necessary back-to-back is enable, the pretrigger is produced by ADC conversion ack*/    
    pPDB_Config->u8Ch0Back2BackEn   =    0x02; /*!<enable channel 0 back-to-back mode */ 
    pPDB_Config->u8Ch1Back2BackEn   =    0x02; /*!<enable channel 1 back-to-back mode */                 
    pPDB_Config->bTriggerSource     =    0x08; /*!<trigger source is from FTM0 */
    PDB_Init(pPDB_Config);                     /*!<init PDB module */
    
    /* FTM module configuration */
    /* FTM generates trigger to PDB at (1000/16)=62.5us */
    //FTM_PinConfig(); /*!<FTM ouput pin configuration*/   
    FTM_PWMInit(FTM0, FTM_PWMMODE_COMBINE, FTM_PWM_LOWTRUEPULSE); /*!<inital FTM to combine mode*/ 
    /* set MOD value */
    FTM_SetModValue(FTM0, core_clk_khz*1000/PWM_FREQ_HZ); /*!<set FTM period is 16KHz, the modulo value is equal (75000/16)-1 */
    FTM_PWMDeadtimeSet(FTM0, 2, 0x13);/*!<set channln and channel n-1 dead time, 1us*/   
    /* set clock source and start the counter */
    FTM_ClockSet(FTM0, FTM_CLOCK_SYSTEMCLOCK, FTM_CLOCK_PS_DIV1);
    /* set the duty cycle, note: only fit for combine mode */
    FTM_SetDutyCycleCombine(FTM0, FTM_CHANNEL_CHANNEL1, 50);
    FTM_SetDutyCycleCombine(FTM0, FTM_CHANNEL_CHANNEL3, 50);    
    FTM_SetDutyCycleCombine(FTM0, FTM_CHANNEL_CHANNEL5, 50);
    FTM0->EXTTRIG = 0x10;   /*!<trigger source is from FTM0 channel 0 trigger */    

	/* wait until rtx_filt_buffers are initialized  */
 	while(swTimerCNTR <  ((350*PWM_FREQ_HZ)/1000));
 	
 	while(swTimerCNTR <  ((350*PWM_FREQ_HZ)/1000));
    
 	rton.W16 &= ~0xF;
      
	while(1)
	{
#if defined(USE_TERMINAL)    
        if(u8CycleTimes >= (gu8RdCounter + 1 + MAX_NO_SAMPLES)% MAX_NO_SAMPLES )
        {
            /* print RT1 to RT4 ADC results */
            printf("%x,%x,%x,%x\n",u16Result0A[gu8RdCounter], u16Result1A[gu8RdCounter],        
                 u16Result0B[gu8RdCounter], u16Result1B[gu8RdCounter++]);
            if(gu8RdCounter >= MAX_NO_SAMPLES)
            {
                gu8RdCounter = 0;
            }
        }
#endif 
        
		/* idle loop, waiting for timeout */
		while (rton.Bits.idleloop == 1);
        		
		/* Timing of LED flashing */			
        timeout100MS = timeout / (_100MS);
       
		switch(timeout100MS)
		{
		case 0:
            RT1_3_LED_On();
            RT2_4_LED_Off();
            RT_LEDs_On();  /* turn on the related pair of LEDs */
			break;
		case 1:
			if(rton.W16 & 0xf)
            {
                /* turn off all LEDs */
                RT_LEDs_OffAll();
            }
            break;
		case 2:
            RT1_3_LED_Off();
            RT2_4_LED_On();
			RT_LEDs_On();  /* turn on the related pair of LEDs */		
            break;
		case 3:
			if(rton.W16 & 0xf)
            {              
                /* turn off all LEDs */
                RT_LEDs_OffAll();
            }
			break;
		default:
			break;
		}
		rton.Bits.idleloop = 1;   
	} 
}
/*****************************************************************************//*!
*
* @brief FTM_PinConfig
*        
* @param none  
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void FTM_PinConfig(void)
{
    /*configure FTM0 channel0~Channel5 output pin */
    PORTC->PCR[1]  = PORT_PCR_MUX(0x04);    /*!<FTM0_CH0 output enable*/
    PORTE->PCR[25] = PORT_PCR_MUX(0x03);    /*!<FTM0_CH1 output enable*/
    PORTC->PCR[3]  = PORT_PCR_MUX(0x04);    /*!<FTM0_CH2 output enable*/
    PORTC->PCR[4]  = PORT_PCR_MUX(0x04);    /*!<FTM0_CH3 output enable*/
    PORTD->PCR[4]  = PORT_PCR_MUX(0x04);    /*!<FTM0_CH4 output enable*/
    PORTD->PCR[5]  = PORT_PCR_MUX(0x04);    /*!<FTM0_CH5 output enable*/    
}

/*****************************************************************************//*!
*
* @brief ADC0 module task
*        
* @param none  
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/

void ADC0_Task(void)
{
  
    if((ADC0->SC1[0] & ADC_SC1_COCO_MASK)==ADC_SC1_COCO_MASK)
    {    
        u16Result0A[u8CycleTimes] = ADC0->R[0];
       // GPIOD->PTOR = (1<<7);     
        
    }
    if((ADC0->SC1[1] & ADC_SC1_COCO_MASK)==ADC_SC1_COCO_MASK)
    {    
        u16Result0B[u8CycleTimes] = ADC0->R[1];
       // GPIOD->PTOR = (1<<6);

    }  
}

/*****************************************************************************//*!
*
* @brief ADC1 module task
*        
* @param none  
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/

void ADC1_Task(void)
{
  
    
    if((ADC1->SC1[0] & ADC_SC1_COCO_MASK)==ADC_SC1_COCO_MASK)
    {    
        u16Result1A[u8CycleTimes] = ADC1->R[0];
    }
    if((ADC1->SC1[1] & ADC_SC1_COCO_MASK)==ADC_SC1_COCO_MASK)
    {    
        u16Result1B[u8CycleTimes] = ADC1->R[1];
        
        //GPIOD->PTOR = (1<<7);     
        RT_Check(u16Result0A[u8CycleTimes], u16Result1A[u8CycleTimes],        
                 u16Result0B[u8CycleTimes], u16Result1B[u8CycleTimes]);
        u8CycleTimes++;
        if(u8CycleTimes >= MAX_NO_SAMPLES)
        {
            u8CycleTimes = 0;
        }        
    }       
}


/*****************************************************************************//*!
*
* @brief Turn on the related LEDs per finger touch
*        
* @param none  
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/

void RT_LEDs_On(void)
{   
    if(rton.Bits.rt1on)  
    {
       RT_LEDs_OffAll();           
       LED0_On(); 
       LED1_On();        
    }
    if(rton.Bits.rt2on)
    {
       RT_LEDs_OffAll();           
       LED2_On(); 
       LED3_On();                 
    }
    if(rton.Bits.rt3on)
    {
       RT_LEDs_OffAll();           
       LED4_On(); 
       LED5_On();                 
    }
    if(rton.Bits.rt4on)  
    {
       RT_LEDs_OffAll();           
       LED6_On(); 
       LED7_On();                 
    }    
}


/*****************************************************************************//*!
*
* @brief Turn off all RT LEDs.
*        
* @param none  
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/

void RT_LEDs_OffAll(void)
{
   LED0_Off(); 
   LED1_Off();                 
   LED2_Off(); 
   LED3_Off();                 
   LED4_Off(); 
   LED5_Off();                 
   LED6_Off(); 
   LED7_Off();                               
}


/*****************************************************************************//*!
*
* @brief Turn on all RT1 &3  LEDs.
*        
* @param none  
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void RT1_3_LED_On()
{
    LED0_On();
    LED1_On();
    LED4_On();
    LED5_On();
}

/*****************************************************************************//*!
*
* @brief Turn off all RT1 &3  LEDs.
*        
* @param none  
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void RT1_3_LED_Off()
{
    LED0_Off();
    LED1_Off();
    LED4_Off();
    LED5_Off();    
}


/*****************************************************************************//*!
*
* @brief Turn off all RT2 &4  LEDs.
*        
* @param none  
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void RT2_4_LED_Off()
{
    LED2_Off();
    LED3_Off();
    LED6_Off();
    LED7_Off();    
}

/*****************************************************************************//*!
*
* @brief Turn on all RT2 &4  LEDs.
*        
* @param none  
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void RT2_4_LED_On()
{
    LED2_On();
    LED3_On();
    LED6_On();
    LED7_On();    
}
            



/*****************************************************************************//*!
*
* @brief check 4 thermistors to see if fingers are close to them or left 
*  from them. It is called by ADC tasks at ~16KHz (PWM frequency), i.e. 62.5us.
*        
* @param none  
*
* @return none 
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void RT_Check(int16_t i16Rt1, int16_t i16Rt2, int16_t i16Rt3, int16_t i16Rt4)
{
    rt1 = i16Rt1;
    rt2 = i16Rt2;
    rt3 = i16Rt3;
    rt4 = i16Rt4;
        
  /* Execute FIR filter on ADC samples */
    arm_fir_q15(&S1, &i16Rt1, &rt1_filt, 1);    
 	arm_fir_q15(&S2, &i16Rt2, &rt2_filt, 1);
 	arm_fir_q15(&S3, &i16Rt3, &rt3_filt, 1);
 	arm_fir_q15(&S4, &i16Rt4, &rt4_filt, 1);	
	
	/*Reset SWSamplerCNT*/
	if(swDiffCNTR >= DIFF_TS)
	{
		/* evaluate delta */
		delta_rt1 = rt1_filt - rt1_filt_buff[i_delay];
		delta_rt2 = rt2_filt - rt2_filt_buff[i_delay];
		delta_rt3 = rt3_filt - rt3_filt_buff[i_delay];
		delta_rt4 = rt4_filt - rt4_filt_buff[i_delay];
		
		/* store samples in the buffer */
		rt1_filt_buff[i_sample] = rt1_filt;
		rt2_filt_buff[i_sample] = rt2_filt;
		rt3_filt_buff[i_sample] = rt3_filt;
		rt4_filt_buff[i_sample] = rt4_filt;
		if(++i_sample >= BUFF_SIZE) i_sample = 0;
		if(++i_delay >= BUFF_SIZE) i_delay = 0;

		/* detect rt ON/OFF */
		/* Three consequent samples has to be higher than threshold limit */
		/* rt1 ON filter */
		if(delta_rt1 < RT1ONTH)
		{
			rt_filter_on+=0x1;
			if((rt_filter_on&0xf)>=0x3)
			{
				rt_filter_on&=~0xf;
				rton.Bits.rt1on = 1;
			}
		}
		else
		{
			rt_filter_on&=~0xf;	
		}
		/* rt1 OFF filter */
		if(delta_rt1>RT1OFFTH)
		{
			rt_filter_off+=0x1;
			if((rt_filter_off&0xf)>=0x3)
			{
				rt_filter_off&=~0xf;
				rton.W16 &= ~0xf;
			}			
		}
		else
		{
			rt_filter_off&=~0xf;	
		}
		
		/* rt2 ON filter */
		if(delta_rt2<RT2ONTH)
		{
			rt_filter_on+=0x10;
			if((rt_filter_on&0xf0)>=0x30)
			{
				rt_filter_on&=~0xf0;
				rton.Bits.rt2on = 1;
			}
		}
		else
		{
			rt_filter_on&=~0xf0;	
		}
		/* rt2 OFF filter */
		if(delta_rt2>RT2OFFTH)
		{
			rt_filter_off+=0x10;
			if((rt_filter_off&0xf0)>=0x30)
			{
				rt_filter_off&=~0xf0;
				rton.W16 &= ~0xf;
			}			
		}
		else
		{
			rt_filter_off&=~0xf0;	
		}
		
		/* rt3 ON filter */
		if(delta_rt3<RT3ONTH)
		{
			rt_filter_on+=0x100;
			if((rt_filter_on&0xf00)>=0x300)
			{
				rt_filter_on&=~0xf00;
				rton.Bits.rt3on = 1;
			}
		}
		else
		{
			rt_filter_on&=~0xf00;	
		}
		/* rt3 OFF filter */
		if(delta_rt3>RT3OFFTH)
		{
			rt_filter_off+=0x100;
			if((rt_filter_off&0xf00)>=0x300)
			{
				rt_filter_off&=~0xf00;
				rton.W16 &= ~0xf;
			}			
		}
		else
		{
			rt_filter_off&=~0xf00;	
		}

		/* rt4 ON filter */
		if(delta_rt4<RT4ONTH)
		{
			rt_filter_on+=0x1000;
			if((rt_filter_on&0xf000)>=0x3000)
			{
				rt_filter_on&=~0xf000;
				rton.Bits.rt4on = 1;
			}
		}
		else
		{
			rt_filter_on&=~0xf000;	
		}
		/* rt4 OFF filter */
		if(delta_rt4>RT4OFFTH)
		{
			rt_filter_off+=0x1000;
			if((rt_filter_off&0xf000)>=0x3000)
			{
				rt_filter_off&=~0xf000;
				rton.W16 &= ~0xf;
			}			
		}
		else
		{
			rt_filter_off&=~0xf000;	
		}
			
		/* reset counter */
		swDiffCNTR = 0;
	}
	else
	{
		swDiffCNTR++;
	}
	
    /* Reset SWTimerCNTR every 400ms */
    /* Software timer is used to control LED flashing intervals */
	if(swTimerCNTR >= SWTMR_TOUT)
	{
		swTimerCNTR = 0;
	}
	else
	{
		swTimerCNTR++;
	}
	
	if(swTimerCNTR == timeout)
	{
		rton.Bits.idleloop = 0;
		timeout += _100MS;                      /* time out event every 100 ms */
		if(timeout >= SWTMR_TOUT) timeout = 0;
	}    
}

